
# q-expansion of newform 425.2.e.e, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 425
weight = 2
poly_data = [1, 0, 278, 0, 519, 0, 364, 0, 119, 0, 18, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[3, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[6, -97, 29, 262, -62, 414, -58, 186, -14, 33, -1, 2], 20], [[-6, -97, -29, 262, 62, 414, 58, 186, 14, 33, 1, 2], 20], [[-22, 6, -653, 29, -776, -62, -314, -58, -52, -14, -3, -1], 20], [[22, 6, 653, 29, 776, -62, 314, -58, 52, -14, 3, -1], 20], [[0, 168, 0, 507, 0, 479, 0, 191, 0, 33, 0, 2], 10], [[2, 0, 388, 0, 531, 0, 249, 0, 47, 0, 3, 0], 10], [[0, -502, 0, -1133, 0, -906, 0, -324, 0, -52, 0, -3], 10], [[37, 552, 1118, 1143, 1486, 906, 684, 324, 127, 52, 8, 3], 20], [[37, -552, 1118, -1143, 1486, -906, 684, -324, 127, -52, 8, -3], 20]]

hecke_ring_character_values = [[52, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [326, [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, -1, 0, 0, 1, -1, 1, 0, 0, -1], [0, 1, 1, 1, 0, 0, -1, 1, -1, 1, 0, 0], [-2, 0, 1, 0, 0, 0, 0, 0, -1, 0, 1, 1], [-1, -1, -1, 0, 1, -1, 1, 0, 1, 0, 0, -1], [0, -2, 0, -1, -1, 0, 0, -2, 0, -1, 1, -1], [-1, 1, 1, 3, 0, 0, -1, 2, -1, 1, 0, 2], [0, 0, 0, 0, -1, 0, 0, 0, 0, 0, -1, 0], [-1, -1, 1, 0, 1, 0, 0, -1, -1, -1, 0, 0], [0, 1, 0, 0, 1, 2, 0, -2, 1, 0, -1, 0], [0, -1, 0, -1, 0, 0, 2, -2, 1, 0, 0, -1], [0, -2, 0, -1, -1, -1, -1, 2, 0, 1, 0, 0], [-4, 0, 2, 2, -2, -1, 1, 0, -2, 0, 0, 0], [0, -2, 0, -1, -1, 0, 0, -5, 0, -3, 1, -1], [0, -2, 0, -3, -3, 1, 1, -4, 0, 1, 1, -1], [-2, 2, -1, -2, 0, 0, 2, 0, -2, -1, 0, -2], [0, 0, 0, -1, 1, 0, 0, 0, 4, 0, -1, -1], [-1, 0, -1, 0, 1, 1, 0, -2, 0, 1, -1, 0], [2, 5, -4, 0, 3, -2, 0, 4, 5, 4, -1, 0], [-1, 1, -1, 2, 0, 0, 1, 0, -1, -1, 0, -2], [0, -4, 0, 0, 0, 1, 1, -8, 0, -2, 2, -2], [-2, 0, -3, -3, 3, -1, 1, 0, 4, 0, -1, -1], [4, -1, 0, 0, -2, 2, 0, 2, -1, 0, 4, 0], [2, 0, -1, 0, 0, -1, 1, 0, 3, 0, -4, -4], [0, 0, 2, -3, 3, -1, 1, 0, -2, 0, 0, 0], [-3, -1, -2, 0, 2, -1, 0, -2, -1, 2, -2, 0], [-3, -3, -1, -3, 0, 0, 2, 1, 3, -1, 0, -1], [1, 0, 0, 0, 0, 0, 2, -3, 0, 0, 0, -4], [0, -2, 0, -2, -2, 1, 1, 8, 0, -5, -2, 2], [-4, -6, 1, 0, -4, -1, 0, -3, -6, -1, 4, 0], [10, 0, 1, 2, -2, -1, 1, 0, 3, 0, 0, 0], [3, -1, 2, 0, -2, 2, 0, 1, -1, -2, 5, 0], [-1, 0, 0, 0, 0, -1, 1, 0, 0, 0, -1, -1], [0, 4, 0, 5, 5, -2, -2, 4, 0, 4, -1, 1], [3, 0, 3, 3, -3, 1, -1, 0, -8, 0, 0, 0], [-1, 3, 0, -2, 0, 0, -4, 5, -3, 0, 0, -1], [2, 5, -2, 0, 3, -1, 0, 3, 5, 2, -6, 0], [-2, -1, 0, 0, -1, -2, 0, 0, -1, 0, -1, 0], [0, 4, 0, 6, 6, -4, -4, 6, 0, 3, -3, 3], [3, 5, 0, 3, 0, 0, 0, -3, -5, 0, 0, 1], [6, 0, 1, -1, 1, 1, -1, 0, 0, 0, 4, 4], [5, -4, 0, -3, 0, 0, 2, -7, 4, 0, 0, -3], [-4, -1, -4, -2, 0, 0, -2, 6, 1, -4, 0, 0], [3, -3, 0, 0, -1, 5, 0, -2, -3, 0, 0, 0], [6, 0, 4, 4, 0, 0, -5, -1, 0, 4, 0, -2], [0, 8, 0, 2, 2, -5, -5, 4, 0, 4, -5, 5], [3, 5, 0, 0, 0, 0, 0, -3, -5, 0, 0, -1], [0, -1, 0, 4, 4, -1, -1, 0, 0, 2, -1, 1], [10, 2, 3, 0, -1, 2, 0, 8, 2, -3, -3, 0], [-8, 0, 0, -2, 2, 0, 0, 0, 2, 0, -1, -1], [-9, 0, 0, 0, 0, 0, 0, -9, 0, 0, 0, 0], [-2, 0, 6, 1, -1, 4, -4, 0, -8, 0, 4, 4], [0, -1, 0, 3, 3, -3, -3, 2, 0, 4, -2, 2], [0, -2, 0, 3, 3, 0, 0, 2, 0, -2, 3, -3], [-8, -5, 3, 0, -2, 0, 0, -8, -5, -3, 6, 0], [-2, 0, -1, 2, -2, 0, 0, 0, -6, 0, 5, 5], [-7, -4, 0, 0, 5, 0, 0, -7, -4, 0, -1, 0], [0, -4, 0, 1, 1, 3, 3, -5, 0, -2, 0, 0], [-10, 1, 2, 1, 0, 0, 3, 7, -1, 2, 0, 4], [-12, 0, 0, -2, 2, -1, 1, 0, 6, 0, -1, -1], [6, 0, 6, 1, -1, 0, 0, 0, 0, 0, 2, 2], [4, 4, -3, 0, 0, 0, 0, 4, 4, 3, -3, 0], [5, -3, -3, -5, 0, 0, 2, -7, 3, -3, 0, -5], [-5, 3, 7, -1, 0, 0, -2, 7, -3, 7, 0, 5], [0, 0, 0, 2, 2, -1, -1, 8, 0, 2, 0, 0], [-4, -11, 3, 0, -4, 4, 0, -8, -11, -3, 10, 0], [-1, 2, 6, 1, 0, 0, -3, 4, -2, 6, 0, 3], [0, -8, 0, -6, -6, 0, 0, -9, 0, -3, 2, -2], [-1, 0, -6, 0, 0, -3, 3, 0, 8, 0, -1, -1], [0, -12, 0, -2, -2, -2, -2, 0, 0, -3, 1, -1], [-5, -3, 0, 0, 0, 0, 0, 5, 3, 0, 0, -7], [-12, 0, -4, -3, 3, -4, 4, 0, 1, 0, -1, -1], [3, -9, 3, 0, 2, 1, 0, 2, -9, -3, 2, 0], [0, 0, 0, 3, 3, 0, 0, 10, 0, -7, 1, -1], [0, 10, 0, 6, 6, -1, -1, 10, 0, 5, -6, 6], [6, -1, 1, 8, 0, 0, -2, -4, 1, 1, 0, 0], [-8, -9, -4, -13, 0, 0, 8, 0, 9, -4, 0, -7], [1, 0, -9, -6, 6, 1, -1, 0, 8, 0, 1, 1], [0, 0, 2, 11, 0, 0, -5, 5, 0, 2, 0, 0], [-6, 0, -3, -9, 9, -3, 3, 0, 5, 0, -1, -1], [-5, 5, 6, 3, 0, 0, -2, 7, -5, 6, 0, -2], [0, 4, 0, -8, -8, 2, 2, 0, 0, -5, 1, -1], [1, -6, -1, 0, -7, -1, 0, 2, -6, 1, -3, 0], [14, 0, -1, -3, 3, -2, 2, 0, 10, 0, -8, -8], [6, -8, -2, -1, 0, 0, -2, -4, 8, -2, 0, -13], [0, 13, 0, 3, 3, -4, -4, 10, 0, 8, -7, 7], [0, -8, 0, -7, -7, 6, 6, -4, 0, 2, 3, -3], [-8, 0, 1, -3, 3, 0, 0, 0, -4, 0, 3, 3], [0, -2, 0, -3, -3, -1, -1, 8, 0, 0, 3, -3], [4, 2, 5, 0, -13, 1, 0, 3, 2, -5, 0, 0], [6, -1, -1, -3, 0, 0, 1, -7, 1, -1, 0, -10], [0, -4, 0, 3, 3, 1, 1, -8, 0, -8, -1, 1], [0, 8, 3, 11, 0, 0, -2, 2, -8, 3, 0, 8], [7, -10, 2, -5, 0, 0, -1, -6, 10, 2, 0, -6], [3, 0, -2, 1, -1, -4, 4, 0, 14, 0, -1, -1], [-5, 4, 3, 12, 0, 0, -8, 13, -4, 3, 0, 10], [6, 0, -2, -1, 1, -4, 4, 0, -4, 0, -2, -2], [10, 6, -2, 0, 9, -4, 0, 14, 6, 2, -9, 0], [2, 6, -2, 0, -4, 5, 0, -3, 6, 2, 2, 0], [-12, 0, -5, -5, 5, -5, 5, 0, -1, 0, 1, 1], [0, 14, 0, 0, 0, 0, 0, 2, 0, 1, -2, 2], [0, 2, 0, 9, 9, 1, 1, 0, 0, 2, -2, 2], [-3, 9, 2, 7, 0, 0, -4, 7, -9, 2, 0, 16], [-12, 0, -1, 1, -1, 1, -1, 0, 2, 0, 1, 1], [0, -18, 0, -9, -9, 2, 2, 2, 0, -8, 4, -4], [0, 8, 0, 9, 9, -1, -1, -13, 0, 1, 0, 0], [-18, 0, -5, -5, 5, -6, 6, 0, 2, 0, -3, -3], [-13, 14, 3, 14, 0, 0, -6, 19, -14, 3, 0, 12], [5, 9, -1, 0, 13, -4, 0, 9, 9, 1, -12, 0], [-7, 0, 4, 12, -12, 7, -7, 0, -20, 0, 11, 11], [-10, 2, 0, 0, -3, 2, 0, -12, 2, 0, 5, 0], [0, -1, 1, 5, 0, 0, -7, 7, 1, 1, 0, 8], [0, 10, 0, -2, -2, -5, -5, 10, 0, 1, -1, 1], [-2, 11, -1, 0, 7, -6, 0, 4, 11, 1, -7, 0], [-6, 12, -2, 0, 18, -3, 0, -3, 12, 2, -6, 0], [14, 0, -7, 0, 0, -1, 1, 0, 10, 0, -6, -6], [1, -4, 2, -3, 0, 0, 4, -5, 4, 2, 0, -5], [-10, 0, 7, 6, -6, 6, -6, 0, -24, 0, 7, 7], [0, 0, 0, -1, -1, -4, -4, 11, 0, 6, 3, -3], [-8, -2, 0, -6, 0, 0, -2, 10, 2, 0, 0, 4], [8, 6, -7, 0, 0, -6, 0, 14, 6, 7, -2, 0], [4, -4, 2, 0, -4, 1, 0, 3, -4, -2, 14, 0], [11, -6, 1, 1, 0, 0, 3, -14, 6, 1, 0, -3], [10, 0, 11, 3, -3, 1, -1, 0, -12, 0, -1, -1], [-1, 5, -3, 0, 0, -8, 0, 7, 5, 3, -6, 0], [4, 10, -4, 0, 6, 2, 0, 2, 10, 4, -13, 0], [0, 0, 1, 9, -9, 0, 0, 0, 0, 0, 3, 3], [0, -10, 0, -2, -2, 5, 5, -19, 0, -8, 1, -1], [0, -4, 0, -1, -1, -3, -3, 6, 0, 6, -5, 5], [9, -7, 1, 0, -9, 2, 0, 7, -7, -1, 6, 0], [4, 9, -6, 0, -3, -3, 0, 7, 9, 6, -10, 0], [0, -2, 0, -5, -5, -1, -1, -6, 0, 5, -3, 3], [0, 0, -2, -1, 1, 1, -1, 0, -7, 0, -4, -4], [-14, 0, 4, 1, -1, -7, 7, 0, -1, 0, -4, -4], [0, 11, 0, 8, 8, -10, -10, 16, 0, 11, -5, 5], [16, 2, -2, 0, 6, 0, 0, 16, 2, 2, -5, 0], [0, -16, 0, -5, -5, 2, 2, -17, 0, -8, 9, -9], [2, 0, 7, -6, 0, 0, 0, -2, 0, 7, 0, -4], [-6, 4, -8, 0, -2, 4, 0, -10, 4, 8, 5, 0], [0, -10, 8, 0, -16, 4, 0, -4, -10, -8, 6, 0], [-12, 10, 3, 6, 0, 0, -3, 15, -10, 3, 0, 8], [-7, 7, 1, 13, 0, 0, -5, 12, -7, 1, 0, 14], [-1, 0, -11, -8, 8, -3, 3, 0, 8, 0, 3, 3], [4, -2, -6, -12, 0, 0, 8, -12, 2, -6, 0, -13], [3, -13, 3, 0, -15, 6, 0, -3, -13, -3, 9, 0], [7, 0, -2, 5, 0, 0, -2, -5, 0, -2, 0, 1], [0, -4, 0, 2, 2, 2, 2, -8, 0, 6, 4, -4], [-16, 0, 8, 1, -1, 4, -4, 0, -8, 0, 4, 4], [-5, 4, -1, -9, 0, 0, 0, 5, -4, -1, 0, 9], [5, -1, 1, 0, -1, 6, 0, -1, -1, -1, 5, 0], [-6, 0, -3, 5, -5, -4, 4, 0, 0, 0, -4, -4], [8, 6, 5, 0, 3, -6, 0, 14, 6, -5, -8, 0], [6, 1, 2, 1, 0, 0, -7, 1, -1, 2, 0, 10], [12, -15, -11, -7, 0, 0, 5, -17, 15, -11, 0, -9], [-2, 0, -1, 2, -2, -4, 4, 0, 18, 0, -3, -3], [6, -9, -9, -11, 0, 0, 2, -8, 9, -9, 0, 1], [0, 10, 0, -4, -4, -3, -3, 9, 0, -3, 1, -1], [-4, -2, -10, -1, 0, 0, 8, -4, 2, -10, 0, -9], [-15, -4, 0, 0, 7, -5, 0, -10, -4, 0, 10, 0], [4, 0, 0, -4, 4, 3, -3, 0, -7, 0, 7, 7], [0, -10, 0, -8, -8, 7, 7, -24, 0, -3, 9, -9], [0, 8, 0, 2, 2, -5, -5, 4, 0, 2, -7, 7], [0, 4, 0, 2, 2, -1, -1, -19, 0, 1, -3, 3], [1, -4, -2, 0, 3, -1, 0, 2, -4, 2, -7, 0], [-10, -10, 2, 0, -12, -2, 0, -8, -10, -2, -3, 0], [-6, 4, 3, -6, 0, 0, 4, 2, -4, 3, 0, -14]]
